#pragma once
#include <SFML/Graphics.hpp>
#include <unordered_map>
#include <string>
#include <fstream>
namespace zle
{
	static const std::string ZLE_VERSION = "#1.3";
	class TileMap : public sf::Transformable, public sf::Drawable
	{
		mutable sf::VertexArray arr;
		mutable sf::VertexBuffer buff;
		sf::Vector2u mapSize;
		sf::Vector2u tileSize;
		sf::Vector2u textureTileSize;
		sf::Color overAllColor;
		sf::Texture* texture;
		unsigned int textureGap;
		sf::Color mapColor;
		mutable bool updateBeforeDraw;
		std::vector<std::vector<unsigned short>> mapData;
		void InternalUpdate()
		{
			//resize buffer
			mapData.resize(mapSize.x);
			for (int i = 0; i < mapSize.x; i++)
				mapData[i].resize(mapSize.y);
		}
		void DrawUpdate() const
		{
			updateBeforeDraw = 0;
			arr.setPrimitiveType(sf::Triangles);
			arr.clear();
			arr.resize(6 * mapSize.x * mapSize.y);
			if (sf::VertexBuffer::isAvailable())
			{
				buff.setPrimitiveType(sf::Triangles);
				buff.setUsage(sf::VertexBuffer::Dynamic);
			}
			//setup data
			for (int i = 0; i < mapSize.x; i++)
				for (int j = 0; j < mapSize.y; j++)
				{
					sf::Vector2f position = sf::Vector2f(tileSize.x * i, tileSize.y * j);
					unsigned int v = (j + mapSize.y * i) * 6;

					arr[v].position = position;
					arr[v].color = overAllColor;
					arr[v].texCoords = sf::Vector2f(mapData[i][j] * textureTileSize.x + textureGap * mapData[i][j], 0);

					arr[v + 1].position = position + sf::Vector2f(tileSize.x, 0);
					arr[v + 1].color = overAllColor;
					arr[v + 1].texCoords = sf::Vector2f(mapData[i][j] * textureTileSize.x + textureTileSize.x + textureGap * mapData[i][j], 0);

					arr[v + 2].position = position + (sf::Vector2f)tileSize;
					arr[v + 2].color = overAllColor;
					arr[v + 2].texCoords = sf::Vector2f(mapData[i][j] * textureTileSize.x + textureTileSize.x + textureGap * mapData[i][j], textureTileSize.y);

					arr[v + 3].position = position + sf::Vector2f(0, tileSize.y);
					arr[v + 3].color = overAllColor;
					arr[v + 3].texCoords = sf::Vector2f(mapData[i][j] * textureTileSize.x + textureGap * mapData[i][j], textureTileSize.y);

					arr[v + 4] = arr[v];
					arr[v + 5] = arr[v + 2];
				}

			//update
			if (sf::VertexBuffer::isAvailable() && arr.getVertexCount() > 0)
			{
				if (buff.getVertexCount() != arr.getVertexCount())
					buff.create(arr.getVertexCount());
				buff.update(&arr[0]);
			}
		}
		template<typename T>
		static void Save(std::ofstream& s, T var)
		{
			for (int i = 0; i < sizeof(T); i++)
				s << ((char*)(&var))[i];
		}
		template<typename T>
		static void Save(std::string& s, T var)
		{
			for (int i = 0; i < sizeof(T); i++)
				s += ((char*)(&var))[i];
		}
		template<typename T>
		void getNumber(sf::InputStream& input, T& t)
		{
			input.read((void*)&t, sizeof(T));
		}
	public:

		TileMap()
			: texture(nullptr), textureGap(0), updateBeforeDraw(0)
		{
			overAllColor = sf::Color::White;
			mapSize = sf::Vector2u(1, 1);
			tileSize = sf::Vector2u(1, 1);
			textureTileSize = Vector2u(1, 1);
		}
		~TileMap() {}

		void saveToFile(const std::string& fileName)
		{
			std::ofstream save1;
			save1.open(fileName);
			if (save1.is_open())
			{
				Save(save1, getTileCount().x);
				Save(save1, getTileCount().y);
				for (int i = 0; i < getTileCount().x; i++)
					for (int j = 0; j < getTileCount().y; j++)
					{
						Save(save1, getTileID(sf::Vector2u(i, j)));
					}
				save1.close();
			}
		}
		void saveToMemory(std::string& memory)
		{
			memory = "";
			Save(memory, getTileCount().x);
			Save(memory, getTileCount().y);
			for (int i = 0; i < getTileCount().x; i++)
				for (int j = 0; j < getTileCount().y; j++)
				{
					Save(memory, getTileID(sf::Vector2u(i, j)));
				}
		}
		void loadFromMemory(const char* data, Uint64 Size)
		{
			sf::MemoryInputStream stream;
			stream.open(data, Size);
			sf::Vector2u size;
			getNumber(stream, size.x);
			getNumber(stream, size.y);

			setTileCount(size);
			for (int i = 0; i < getTileCount().x; i++)
				for (int j = 0; j < getTileCount().y; j++)
				{
					unsigned short var;
					getNumber(stream, var);
					setTileID(sf::Vector2u(i, j), var);
				}
		}
		void loadFromStream(sf::FileInputStream& stream)
		{
			sf::Vector2u size;
			getNumber(stream, size.x);
			getNumber(stream, size.y);
			setTileCount(size);
			for (int i = 0; i < getTileCount().x; i++)
				for (int j = 0; j < getTileCount().y; j++)
				{
					unsigned short var;
					getNumber(stream, var);
					setTileID(sf::Vector2u(i, j), var);
				}
		}
		void loadFromFile(const std::string& fileName)
		{
			sf::FileInputStream load1;
			if (load1.open(fileName))
			{
				loadFromStream(load1);
			}
		}

		void setTileCount(const sf::Vector2u& mapSize)
		{
			this->mapSize = mapSize;
			InternalUpdate();
		}
		const sf::Vector2u& getTileCount() const
		{
			return mapSize;
		}
		void setTexture(sf::Texture* texture)
		{
			this->texture = texture;
		}
		const sf::Texture* getTexture() const
		{
			return texture;
		}
		void setTileSize(const sf::Vector2u& tileSize)
		{
			this->tileSize = tileSize;
			updateBeforeDraw = 1;
		}
		const sf::Vector2u& getTileSize() const
		{
			return tileSize;
		}
		void setTextureTileSize(const sf::Vector2u& textureTileSize)
		{
			this->textureTileSize = textureTileSize;
			updateBeforeDraw = 1;
		}
		void setTextureGap(unsigned int gap)
		{
			textureGap = gap;
			updateBeforeDraw = 1;
		}
		unsigned int getTextureGap() const
		{
			return textureGap;
		}
		sf::Vector2u getTextureTileSize() const
		{
			return textureTileSize;
		}
		void setColor(sf::Color color)
		{
			overAllColor = color;
			updateBeforeDraw = 1;
		}
		sf::Color getColor() const
		{
			return overAllColor;
		}
		unsigned short getTileID(sf::Vector2u tile) const
		{
			return mapData[tile.x][tile.y];
		}
		void setTileID(sf::Vector2u tile, unsigned short ID)
		{
			mapData[tile.x][tile.y] = ID;
			updateBeforeDraw = 1;
		}
	private:
		virtual void draw(sf::RenderTarget& target, sf::RenderStates states) const
		{
			states.transform *= getTransform();
			states.texture = texture;
			if (updateBeforeDraw)
				DrawUpdate();
			if (sf::VertexBuffer::isAvailable())
				target.draw(buff, states);
			else
				target.draw(arr, states);
		}
	};
	class ParticleSystemEvent
	{
	public:
		enum class Type
		{
			None = 0,
			OnUpdate = 1,
			OnDeath = 1 << 1,
			OnCreate = 1 << 2,
			All = OnUpdate | OnDeath | OnCreate
		};
		Type type = Type::None;
		sf::Vector2f particlePosition = sf::Vector2f();
		float particleRotation = 0;
		sf::Color particleColor = sf::Color();
		sf::Time lifeTime = sf::Time();
		ParticleSystemEvent()
		{

		}
		ParticleSystemEvent(const Type& type, const sf::Vector2f& particlePosition, const float& particleRotation, const sf::Color& particleColor, const sf::Time& lifeTime)
			: type(type), particlePosition(particlePosition), particleRotation(particleRotation), particleColor(particleColor), lifeTime(lifeTime)
		{
		}
		~ParticleSystemEvent()
		{

		}
	};
	class ParticleSystem : public sf::Drawable, public sf::Transformable
	{
	public:
		enum class ParticleType;
	private:
		friend class Level;
		struct Particle : public sf::Transformable
		{
		private:
		public:
			std::vector<sf::Vertex*> vertices;
			sf::Color currentColor;
			sf::Color startColor;
			sf::Color endColor;
			sf::Time lifeTime;
			sf::Vector2f direction;
			sf::Vector2f offset;
			sf::Vector2f force;
			bool inUse;
			Particle()
				:inUse(0), offset(0, 0), direction(0, 0)
			{

			}
			void resetPointCoord()
			{
				switch (vertices.size())
				{
				case 1:
					vertices[0]->texCoords = sf::Vector2f(0, 0);
				case 2: case 3:
				{
					float add = 360.0 / vertices.size();
					for (int i = 0; i < vertices.size(); i++)
					{
						vertices[i]->texCoords = sf::Vector2f(cos((add * i - 45) * 0.0174533), sin((add * i - 45) * 0.0174533));
					}
				}
				break;
				case 4:
					vertices[0]->texCoords = sf::Vector2f(-1, -1);
					vertices[1]->texCoords = sf::Vector2f(1, -1);
					vertices[2]->texCoords = sf::Vector2f(1, 1);
					vertices[3]->texCoords = sf::Vector2f(-1, 1);
					break;
				case 6:
					vertices[0]->texCoords = sf::Vector2f(-1, -1);
					vertices[1]->texCoords = sf::Vector2f(1, -1);
					vertices[2]->texCoords = sf::Vector2f(1, 1);
					vertices[3]->texCoords = sf::Vector2f(-1, 1);
					vertices[4]->texCoords = vertices[0]->texCoords;
					vertices[5]->texCoords = vertices[2]->texCoords;
					break;
				default:
					vertices[0]->texCoords = sf::Vector2f(0, 0);
					break;
				}

			}
			const sf::Vector2f getPoint(int index)
			{
				switch (vertices.size())
				{
				case 1:
					return sf::Vector2f(0, 0);
				case 2: case 3: case 4:
				{
					float add = 360.0 / vertices.size();
					return sf::Vector2f(cos((add * index - 90) * 0.0174533), sin((add * index - 90) * 0.0174533));
					break;
				}
				case 6:
				{
					float add = 360.0 / 4;
					if (index == 4)
						return sf::Vector2f(cos((-90) * 0.0174533), sin((-90) * 0.0174533));
					if (index == 5)
						return sf::Vector2f(cos((add * 2 - 90) * 0.0174533), sin((add * 2 - 90) * 0.0174533));
					return sf::Vector2f(cos((add * index - 90) * 0.0174533), sin((add * index - 90) * 0.0174533));
					break;
				}
				default:
					return sf::Vector2f(0, 0);
				}
			}
			void Update()
			{
				if (vertices.size() == 1)
				{
					vertices[0]->position = getPosition();
					vertices[0]->color = currentColor;
				}
				else
					for (int i = 0; i < vertices.size(); i++)
					{
						vertices[i]->position = getTransform().transformPoint(getPoint(i));
						vertices[i]->color = currentColor;

					}


			}
		};
		std::vector<ParticleSystemEvent> events;
		std::vector<sf::Vertex> arr;
		sf::VertexBuffer buff;
		std::vector<Particle> particles;
		sf::Time updateTime = sf::Time::Zero;
		sf::Time cntDown = sf::Time::Zero;
		int totalParticles = 0;
		sf::Uint8 eventFlags = 0;

		//every particle variable
		sf::Vector2f startPos = sf::Vector2f();
		int maxParticles;
		float lifeSeconds;
		ParticleType type;
		float startRotation;
		float endRotation;
		int spawnCount;
		float spawnSeconds;
		sf::Vector2f constantForce;
		sf::Vector2f startForce;
		float startSize;
		float endSize;
		float startSpeed;
		float endSpeed;
		float fireAngle;
		float fireRotation;
		float minSpeed = 1;
		float maxSpeed = 1;
		sf::Color startColor;
		sf::Color endColor;
		float fading;
		bool randomStartRotation;
		bool createOnUpdate = 1;
		const sf::Texture* texture;
		bool firstUpdate;
		bool keepUpWithFrameRate;
		bool drawNewestOnTop;
		bool useThread;
		bool updateSystem;
		bool returned = 0;
		sf::Time createFor = sf::Time::Zero;
		unsigned int verticesPerParticle() const
		{
			switch (type)
			{
			case ParticleSystem::ParticleType::Points:
				return 1;
				break;
			case ParticleSystem::ParticleType::Lines:
				return 2;
				break;
			case ParticleSystem::ParticleType::Triangles:
				return 3;
				break;
			case ParticleSystem::ParticleType::Quads:
				return 4;
				break;
			case ParticleSystem::ParticleType::QuadsTriangles:
				return 6;
				break;
			default:
				return 1;
				break;
			}
		}
		float getParam(const sf::Time& lifeLeft, float start, float end) const
		{
			float zeroToOne = (lifeSeconds - lifeLeft.asSeconds()) / lifeSeconds;
			return (end - start) * zeroToOne + start;
		}
		void refreshPointers()
		{
			if (!drawNewestOnTop)
				return;
			for (int j = 0; j < particles.size(); j++)
			{
				for (int k = 0; k < verticesPerParticle(); k++)
				{
					particles[j].vertices[k] = &arr[j * verticesPerParticle() + k];
				}
			}
		}
		int findFreeParticle()
		{
			for (int i = 0; i < particles.size(); i++)
			{
				if (!particles[i].inUse)
					return i;
			}
			return -1;
		}
		void setupArray()
		{
			if (drawNewestOnTop)
				return;
			particles.clear();
			arr.clear();
			totalParticles = 0;
			particles.resize(maxParticles);
			arr.resize(maxParticles * verticesPerParticle());
			for (int i = 0; i < particles.size(); i++)
			{
				particles[i].vertices.resize(verticesPerParticle());
				for (int j = 0; j < verticesPerParticle(); j++)
					particles[i].vertices[j] = &arr[i * verticesPerParticle() + j];
			}

		}
		void applyTexture(const sf::Texture& newTexture)
		{
			if (&newTexture == nullptr)
				return;
			sf::Vector2f size = (sf::Vector2f)newTexture.getSize();
			if (type == ParticleType::Points)
			{
				for (int i = 0; i < particles.size(); i++)
				{
					particles[i].vertices[0]->texCoords = sf::Vector2f(size.x / 2, size.y / 2);
				}
			}
			else
				for (int i = 0; i < particles.size(); i++)
				{
					particles[i].resetPointCoord();
					for (int j = 0; j < particles[i].vertices.size(); j++)
					{
						sf::Vector2f& v = particles[i].vertices[j]->texCoords;
						v = sf::Vector2f((v.x + 1) / 2.0 * size.x, (v.y + 1) / 2.0 * size.y);
					}
				}

		}
		void CreateEvent(const ParticleSystemEvent::Type& type, const Particle& particle)
		{
			if ((eventFlags & (int)type) > 0)
				events.emplace_back(type, particle.getPosition(), particle.getRotation(), particle.currentColor, particle.lifeTime);
		}
		void setup()
		{
			if (sf::VertexBuffer::isAvailable())
			{
				buff.setUsage(sf::VertexBuffer::Stream);
				if (type == ParticleType::QuadsTriangles)
					buff.setPrimitiveType(sf::PrimitiveType::Triangles);
				else
					buff.setPrimitiveType((sf::PrimitiveType)type);
			}
			deleteParticles();
			setupArray();
		}
		template<typename T>
		static void Save(std::ofstream& s, T var)
		{
			for (int i = 0; i < sizeof(T); i++)
				s << ((char*)(&var))[i];
			s << " ";
		}
		template<typename T>
		void getNumber(sf::FileInputStream& input, T& t)
		{
			std::string s = "";
			char c;
			while (1)
			{
				input.read(&c, sizeof(char));
				if (c >= '0' && c <= '9' || c == '.' || c == '-')
					s.push_back(c);
				else
					break;
			}
			t = std::stof(s);
		}
		void getString(sf::FileInputStream& input, std::string& str)
		{
			char c;
			while (1)
			{
				input.read(&c, sizeof(char));
				if (c == ' ' || c == '\0' || c == '\n')
					break;
				else
					str.push_back(c);
			}
		}
	public:
		enum class ParticleType
		{
			Points = sf::Points,
			Lines = sf::Lines,
			LineStrip = sf::LineStrip,
			Triangles = sf::Triangles,
			TriangleStrip = sf::TriangleStrip,
			TriangleFan = sf::TriangleFan,
			Quads = sf::Quads,
			QuadsTriangles
		};

		ParticleSystem()
			: maxParticles(100), lifeSeconds(1), type(ParticleType::Triangles), startRotation(0), endRotation(0),
			spawnCount(1), spawnSeconds(0.1), constantForce(0, 0), startForce(0, 0), startSize(2), endSize(2),
			startSpeed(10), endSpeed(10), startColor(255, 255, 255, 255), endColor(255, 255, 255, 255), fading(1),
			randomStartRotation(0), createOnUpdate(1), texture(nullptr), firstUpdate(0), keepUpWithFrameRate(0),
			drawNewestOnTop(1), fireAngle(360), fireRotation(0), updateSystem(1), minSpeed(1), maxSpeed(1)
		{
			setup();
		}
		~ParticleSystem()
		{
		}

		void saveToFile(const std::string& fileName) const
		{
			std::ofstream save1;
			save1.open(fileName);
			if (save1.is_open())
			{
				Save(save1, getConstantForce().x);
				Save(save1, getConstantForce().y);
				Save(save1, (int)getEndColor().r);
				Save(save1, (int)getEndColor().g);
				Save(save1, (int)getEndColor().b);
				Save(save1, (int)getEndColor().a);
				Save(save1, getEndRotation());
				Save(save1, getEndSize());
				Save(save1, getEndSpeed());
				Save(save1, getFading());
				Save(save1, getLifeTime());
				Save(save1, getMaxParticles());
				Save(save1, getParticleType());
				Save(save1, getRandomStartRotation());
				Save(save1, getSpawnAngle());
				Save(save1, getSpawnCount());
				Save(save1, getSpawnRate());
				Save(save1, getSpawnRotation());
				Save(save1, (int)getStartColor().r);
				Save(save1, (int)getStartColor().g);
				Save(save1, (int)getStartColor().b);
				Save(save1, (int)getStartColor().a);
				Save(save1, getStartForce().x);
				Save(save1, getStartForce().y);
				Save(save1, getStartRotation());
				Save(save1, getStartSize());
				Save(save1, getStartSpeed());
				Save(save1, getDrawNewestOnTop());
				Save(save1, getKeepUpWithFrameRate());
				Save(save1, getSpawnPosition().x);
				Save(save1, getSpawnPosition().y);
				Save(save1, getRandomSpeedMultiplierMin());
				Save(save1, getRandomSpeedMultiplierMax());
				save1.close();
			}
		}
		void loadFromStream(sf::FileInputStream& file)
		{
			getNumber(file,constantForce.x);
			getNumber(file,constantForce.y);
			float arr[4];
			getNumber(file, arr[0]);
			getNumber(file, arr[1]);
			getNumber(file, arr[2]);
			getNumber(file, arr[3]);
			endColor = sf::Color(arr[0], arr[1], arr[2], arr[3]);
			getNumber(file, endRotation);
			getNumber(file, endSize);
			getNumber(file, endSpeed);
			getNumber(file, fading);
			getNumber(file, lifeSeconds);
			getNumber(file, maxParticles);
			float tempo;
			getNumber(file, tempo);
			type = (ParticleSystem::ParticleType)tempo;
			getNumber(file, randomStartRotation);
			getNumber(file, fireAngle);
			getNumber(file, spawnCount);
			getNumber(file, spawnSeconds);
			getNumber(file, fireRotation);
			getNumber(file, arr[0]);
			getNumber(file, arr[1]);
			getNumber(file, arr[2]);
			getNumber(file, arr[3]);
			startColor = sf::Color(arr[0], arr[1], arr[2], arr[3]);
			getNumber(file, startForce.x);
			getNumber(file, startForce.y);
			getNumber(file, startRotation);
			getNumber(file, startSize);
			getNumber(file, startSpeed);
			getNumber(file, drawNewestOnTop);
			getNumber(file, keepUpWithFrameRate);
			getNumber(file, startPos.x);
			getNumber(file, startPos.y);
			getNumber(file, minSpeed);
			getNumber(file, maxSpeed);
			setParticleType(type);
		}
		void loadFromFile(const std::string& fileName)
		{
			sf::FileInputStream load1;
			if (load1.open(fileName))
			{
				loadFromStream(load1);
			}
		}

		const float& getSpawnAngle() const
		{
			return fireAngle;
		}
		const float& getSpawnRotation() const
		{
			return fireRotation;
		}
		const unsigned int& getMaxParticles() const
		{
			return maxParticles;
		}
		const float& getLifeTime() const
		{
			return lifeSeconds;
		}
		const ParticleType& getParticleType() const
		{
			return type;
		}
		const float& getStartRotation() const
		{
			return startRotation;
		}
		const float& getEndRotation() const
		{
			return endRotation;
		}
		const unsigned int& getSpawnCount() const
		{
			return spawnCount;
		}
		const float& getSpawnRate() const
		{
			return spawnSeconds;
		}
		const sf::Vector2f& getConstantForce() const
		{
			return constantForce;
		}
		const sf::Vector2f& getStartForce() const
		{
			return startForce;
		}
		const float& getStartSize() const
		{
			return startSize;
		}
		const float& getEndSize() const
		{
			return endSize;
		}
		const float& getStartSpeed() const
		{
			return startSpeed;
		}
		const float& getEndSpeed() const
		{
			return endSpeed;
		}

		const sf::Color& getStartColor() const
		{
			return startColor;
		}
		const sf::Color& getEndColor() const
		{
			return endColor;
		}
		const float& getFading() const
		{
			return fading;
		}
		const bool& getRandomStartRotation() const
		{
			return randomStartRotation;
		}
		const sf::Texture& getTexture() const
		{
			return *texture;
		}
		const int& getActiveParticles() const
		{
			if (!drawNewestOnTop)
				return totalParticles;
			else
				return particles.size();
		}
		const sf::Vector2f& getSpawnPosition() const
		{
			return startPos;
		}
		void setCaptureEvent(bool state, sf::Uint8 type = (sf::Uint8)ParticleSystemEvent::Type::All)
		{
			if (state)
			{
				eventFlags |= type;
			}
			else
			{
				eventFlags &= ~type;
			}
		}
		void deleteParticles()
		{
			particles.clear();
			arr.clear();
			totalParticles = 0;
		}
		bool getDrawNewestOnTop() const
		{
			return drawNewestOnTop;
		}
		bool getKeepUpWithFrameRate() const
		{
			return keepUpWithFrameRate;
		}
		const float& getRandomSpeedMultiplierMin() const
		{
			return minSpeed;
		}
		const float& getRandomSpeedMultiplierMax() const
		{
			return maxSpeed;
		}
		bool pollEvent(ParticleSystemEvent& event)
		{
			if (events.size() > 0)
			{
				event = events.back();
				events.pop_back();
				return 1;
			}
			return 0;
		}
		void setDrawNewestOnTop(bool state)
		{
			drawNewestOnTop = state;

			deleteParticles();
			setupArray();
		}
		void setSpawnAngle(float fireAngle)
		{
			this->fireAngle = fireAngle;
		}
		void setSpawnPosition(const sf::Vector2f& pos)
		{
			this->startPos = pos;
		}
		void setSpawnPosition(const float x, const float y)
		{
			this->startPos = sf::Vector2f(x, y);
		}
		void setSpawnRotation(float rotation)
		{
			this->fireRotation = rotation;
		}
		void setMaxParticles(unsigned int count)
		{
			maxParticles = count;
			setupArray();
		}
		void setLifeTime(float seconds)
		{
			lifeSeconds = seconds;
		}
		void setParticleType(ParticleType type)
		{
			this->type = type;
			if (sf::VertexBuffer::isAvailable())
			{
				if (type == ParticleType::QuadsTriangles)
					buff.setPrimitiveType(sf::PrimitiveType::Triangles);
				else
					buff.setPrimitiveType((sf::PrimitiveType)type);
			}
			deleteParticles();
			setupArray();

		}
		void setStartRotation(float rotation)
		{
			startRotation = rotation;
		}
		void setEndRotation(float rotation)
		{
			endRotation = rotation;
		}
		void setSpawnCount(unsigned int count)
		{
			spawnCount = count;
		}
		void setSpawnRate(float time)
		{
			spawnSeconds = time;
			cntDown = sf::Time::Zero;
			createFor = sf::Time::Zero;
		}
		void setConstantForce(sf::Vector2f force)
		{
			constantForce = force;
		}
		void setStartForce(sf::Vector2f force)
		{
			startForce = force;
		}
		void setStartSize(float size)
		{
			startSize = size;
		}
		void setEndSize(float size)
		{
			endSize = size;
		}
		void setStartSpeed(float speed)
		{
			startSpeed = speed;
		}
		void setEndSpeed(float speed)
		{
			endSpeed = speed;
		}
		void setRandomSpeedMultiplier(float min, float max)
		{
			minSpeed = min;
			maxSpeed = max;
		}
		void setStartColor(const sf::Color& color)
		{
			startColor = color;
		}
		void setEndColor(const sf::Color& color)
		{
			endColor = color;
		}
		void setFading(float percentage)
		{
			fading = percentage;
		}
		void setRandomStartRotation(bool state)
		{
			randomStartRotation = state;
		}
		void setTexture(const sf::Texture& texture)
		{
			applyTexture(texture);
			this->texture = &texture;
		}
		void setCreateOnUpdate(bool createOnUpdate)
		{
			this->createOnUpdate = createOnUpdate;
		}
		void setKeepUpWithFrameRate(bool state)
		{
			this->keepUpWithFrameRate = state;
			createFor = sf::Time::Zero;
		}
		void Clear()
		{
			for (int i = 0; i < particles.size(); i++)
			{
				if (particles[i].inUse)
					particles[i].lifeTime = seconds(0);
			}
			Update(seconds(0));
		}
		void CreateFor(const sf::Time& time)
		{
			createFor = time;
			Create();
		}
		void Create()
		{
			if (drawNewestOnTop)
				for (int j = 0; j < spawnCount; j++)
				{
					if (arr.size() / verticesPerParticle() >= maxParticles && j == 0)
						return;
					else if (arr.size() / verticesPerParticle() >= maxParticles)
						break;
					Particle particle;
					particle.vertices.resize(verticesPerParticle());
					for (int i = 0; i < verticesPerParticle(); i++)
						arr.push_back(sf::Vertex());
					for (int i = 0; i < verticesPerParticle(); i++)
						particle.vertices[i] = &arr[arr.size() - (verticesPerParticle() - i)];
					particle.lifeTime = sf::seconds(lifeSeconds);
					particle.currentColor = startColor;
					particle.startColor = startColor;
					particle.endColor = endColor;
					particle.force = startForce;
					particle.inUse = 1;
					float randomDirection = (float)rand() / RAND_MAX * fireAngle + fireRotation;
					float multiply = ((float)rand() / RAND_MAX) * (maxSpeed - minSpeed) + minSpeed;
					particle.direction = sf::Vector2f(cos(randomDirection * 0.0174533) * multiply, sin(randomDirection * 0.0174533) * multiply);
					particle.setRotation(randomStartRotation ? (float)rand() / RAND_MAX * 360 : 0);
					if (texture != nullptr)
					{
						sf::Vector2f size = (sf::Vector2f)texture->getSize();
						particle.resetPointCoord();
						for (int j = 0; j < particle.vertices.size(); j++)
						{
							sf::Vector2f& v = particle.vertices[j]->texCoords;
							v = sf::Vector2f((v.x + 1) / 2.0 * size.x, (v.y + 1) / 2.0 * size.y);
						}
					}

					particle.offset = startPos;
					particle.setPosition(startPos);
					totalParticles++;
					particle.Update();
					particles.push_back(particle);
					CreateEvent(ParticleSystemEvent::Type::OnCreate, particle);
				}
			else
			{
				for (int j = 0; j < spawnCount; j++)
				{
					int id = findFreeParticle();
					if (id == -1 && j == 0)
						return;
					else if (id == -1)
						break;
					Particle& particle = particles[id];
					particle.lifeTime = sf::seconds(lifeSeconds);
					particle.currentColor = startColor;
					particle.startColor = startColor;
					particle.endColor = endColor;
					particle.force = startForce;
					particle.inUse = 1;

					float randomDirection = (float)rand() / RAND_MAX * fireAngle + fireRotation;
					float multiply = ((float)rand() / RAND_MAX) * (maxSpeed - minSpeed) + minSpeed;
					particle.direction = sf::Vector2f(cos(randomDirection * 0.0174533) * multiply, sin(randomDirection * 0.0174533) * multiply);
					particle.setRotation(randomStartRotation ? (float)rand() / RAND_MAX * 360 : 0);
					particle.offset = startPos;
					particle.setPosition(startPos);
					particle.Update();
					totalParticles++;
					CreateEvent(ParticleSystemEvent::Type::OnCreate, particle);
				}
			}
			refreshPointers();
			return;

		}
		void Update(const sf::Time& deltaTime)
		{
			sf::Clock clock;
			if (createOnUpdate)
			{
				cntDown -= deltaTime;

				while (cntDown.asSeconds() <= 0)
				{
					Create();
					cntDown += sf::seconds(spawnSeconds);
					if (!keepUpWithFrameRate)
						break;
				}

			}
			createFor -= deltaTime;
			if (createFor.asSeconds() >= 0)
				Create();
			for (int i = 0; i < particles.size(); i++)
			{
				if (!particles[i].inUse)
					continue;
				//update
				Particle& p = particles[i];
				p.lifeTime -= deltaTime;
			}
			int cnt = 0;
			if (drawNewestOnTop)
			{
				cnt = 0;
				for (int i = 0; i < particles.size(); i++)
				{
					if (!particles[i].inUse)
						continue;
					Particle& p = particles[i];
					if (p.lifeTime.asSeconds() <= 0)
					{
						cnt++;
						CreateEvent(ParticleSystemEvent::Type::OnDeath, p);
					}
					else
						break;
				}
				if (cnt > 0)
				{
					//kill particle
					for (int j = 0; j < verticesPerParticle(); j++)
						arr.erase(arr.begin(), arr.begin() + cnt);
					particles.erase(particles.begin(), particles.begin() + cnt);
					totalParticles -= cnt;
					if (totalParticles < 0)
						totalParticles = 0;
					refreshPointers();
				}
			}
			else
			{
				for (int i = 0; i < particles.size(); i++)
				{
					if (!particles[i].inUse)
						continue;
					Particle& p = particles[i];
					if (p.lifeTime.asSeconds() <= 0)
					{
						CreateEvent(ParticleSystemEvent::Type::OnDeath, p);
						totalParticles--;
						if (totalParticles < 0)
							totalParticles = 0;
						p.inUse = 0;
					}
				}
			}

			for (int i = 0; i < particles.size(); i++)
			{
				if (!particles[i].inUse)
					continue;
				Particle& p = particles[i];
				p.force += constantForce * deltaTime.asSeconds();
				p.offset += (p.direction + p.force) * deltaTime.asSeconds() * getParam(p.lifeTime, startSpeed, endSpeed);
				p.setPosition(p.offset);

				if (!randomStartRotation)
					p.setRotation(getParam(p.lifeTime, startRotation, endRotation));
				p.setScale(sf::Vector2f(1, 1) * getParam(p.lifeTime, startSize, endSize));

				float zeroToOne = (lifeSeconds - p.lifeTime.asSeconds()) / lifeSeconds;
				float fadingT = p.currentColor.a;
				if (zeroToOne > fading && fading < 1)
				{
					zeroToOne -= fading;
					zeroToOne /= (1 - fading);
					fadingT = 255 - 255 * zeroToOne;
				}

				p.currentColor = sf::Color(getParam(p.lifeTime, p.startColor.r, p.endColor.r),
					getParam(p.lifeTime, p.startColor.g, p.endColor.g),
					getParam(p.lifeTime, p.startColor.b, p.endColor.b), fadingT);
				CreateEvent(ParticleSystemEvent::Type::OnUpdate, p);
			}
			for (int i = 0; i < particles.size(); i++)
			{
				if (!particles[i].inUse)
					continue;
				Particle& p = particles[i];
				p.Update();
			}
			{
				if (sf::VertexBuffer::isAvailable() && arr.size() > 0)
				{
					if (buff.getVertexCount() != arr.size())
						buff.create(arr.size());
					buff.update(&arr[0]);
				}
			}
			updateTime = clock.getElapsedTime();
		}
	private:
		virtual void draw(sf::RenderTarget& target, sf::RenderStates states) const
		{
			states.transform *= getTransform();
			states.texture = texture;
			if (arr.size() != 0)
			{
				if (sf::VertexBuffer::isAvailable())
				{
					target.draw(buff, states);
				}
				else
				{
					if (type == ParticleType::QuadsTriangles)
						target.draw(&arr[0], arr.size(), sf::PrimitiveType::Triangles, states);
					else
						target.draw(&arr[0], arr.size(), (sf::PrimitiveType)type, states);
				}

			}
		}
	};
	class Level : public sf::Drawable, public sf::Transformable
	{
		enum class ObjectType
		{
			None,
			Tilemap,
			Sprite,
			Texture,
			Shader_F,
			Shader_V,
			ParticleSystem,
		};

		struct Uniforms
		{
			enum class DataTypes
			{
				Bool,
				Float,
				Int,
				Vec2,
				Vec3,
				Vec4,
				IVec2,
				IVec3,
				IVec4,
			};
			std::vector<DataTypes> types;
			std::vector<std::string> uniformNames;
			unsigned int byteSize;
			static int getTypeSize(DataTypes type)
			{
				switch (type)
				{
				case Uniforms::DataTypes::Bool:
					return 1;
					break;
				case Uniforms::DataTypes::Float:
					return 4;
					break;
				case Uniforms::DataTypes::Int:
					return 4;
					break;
				case Uniforms::DataTypes::Vec2:
					return 8;
					break;
				case Uniforms::DataTypes::Vec3:
					return 12;
					break;
				case Uniforms::DataTypes::Vec4:
					return 16;
					break;
				case Uniforms::DataTypes::IVec2:
					return 8;
					break;
				case Uniforms::DataTypes::IVec3:
					return 12;
					break;
				case Uniforms::DataTypes::IVec4:
					return 16;
					break;
				default:
					break;
				}
				return 0;
			}
			unsigned int getIndex(const std::string& name)
			{
				for (int i = 0; i < uniformNames.size(); i++)
				{
					if (name == uniformNames[i])
					{
						return i;
					}
				}
				return 0;
			}
			DataTypes getUniformType(unsigned int index) const
			{
				return types[index];
			}
			int getOffsetToIndex(int index) const
			{
				if (index < 0)
					return -1;
				int offset = 0;
				for (int i = 0; i < index; i++)
				{
					offset += getTypeSize(types[i]);
				}
				return offset;
			}
			static unsigned char* getUniform(unsigned int offset, std::vector<unsigned char>* data)
			{
				return &((*data)[offset]);
			}
			int getIndexFromName(const std::string& name)
			{
				for (int i = 0; i < uniformNames.size(); i++)
				{
					if (uniformNames[i] == name)
						return i;
				}
				return -1;
			}
			void setUniform(unsigned int index, void* uniformPointer, sf::Shader& shader)
			{
				switch (getUniformType(index))
				{
				case Uniforms::DataTypes::Bool:
					shader.setUniform(uniformNames[index], *((bool*)uniformPointer));
					break;
				case Uniforms::DataTypes::Float:
					shader.setUniform(uniformNames[index], *((float*)uniformPointer));
					break;
				case Uniforms::DataTypes::Int:
					shader.setUniform(uniformNames[index], *((int*)uniformPointer));
					break;
				case Uniforms::DataTypes::Vec2:
					shader.setUniform(uniformNames[index], *((sf::Glsl::Vec2*)uniformPointer));
					break;
				case Uniforms::DataTypes::Vec3:
					shader.setUniform(uniformNames[index], *((sf::Glsl::Vec3*)uniformPointer));
					break;
				case Uniforms::DataTypes::Vec4:
					shader.setUniform(uniformNames[index], *((sf::Glsl::Vec4*)uniformPointer));
					break;
				case Uniforms::DataTypes::IVec2:
					shader.setUniform(uniformNames[index], *((sf::Glsl::Ivec2*)uniformPointer));
					break;
				case Uniforms::DataTypes::IVec3:
					shader.setUniform(uniformNames[index], *((sf::Glsl::Ivec3*)uniformPointer));
					break;
				case Uniforms::DataTypes::IVec4:
					shader.setUniform(uniformNames[index], *((sf::Glsl::Ivec4*)uniformPointer));
					break;
				default:
					break;
				}
			}
		};
		struct RenderStateObject
		{
		private:
			friend class Level;
			friend class Drawable;
			sf::Texture texture;
			std::string textureLocation;
			Uniforms uniforms;
			ObjectType type;
		public:
			RenderStateObject()
			{
				type = ObjectType::None;
			}
			~RenderStateObject()
			{

			}
			const ObjectType& getObjectType() const
			{
				return type;
			}
			const sf::Texture& getTexture() const
			{
				return texture;
			}
			const std::string& getTextureLocation() const
			{
				return textureLocation;
			}
		};
		struct Object : public Drawable
		{
		private:
			friend class Level;
			friend class Drawable;
			sf::Sprite* sprite;
			TileMap* tilemap;
			ParticleSystem* particleSystem;
			sf::Texture* usingTexture = nullptr;
			mutable sf::Shader shader;
			mutable std::vector<unsigned char> uniformFData;
			mutable std::vector<unsigned char> uniformVData;
			ObjectType type;
			sf::Int8 drawingZOrder;
			bool visible = 1;
			sf::BlendMode blendMode;
			RenderStateObject* useVShader;
			RenderStateObject* useFShader;
			sf::Color color;

			bool UseSprite()
			{
				if (tilemap || sprite)
					return 0;
				sprite = new sf::Sprite;
				return 1;
			}
			bool UseTileMap()
			{
				if (tilemap || sprite)
					return 0;
				tilemap = new TileMap;
				return 1;
			}
			bool UseParticleSystem()
			{
				if (tilemap || sprite || particleSystem)
					return 0;
				particleSystem = new ParticleSystem;
				return 1;
			}
			void* getPointer() const
			{
				if (tilemap)
					return tilemap;
				if (sprite)
					return sprite;
				if (particleSystem)
					return particleSystem;
			}
		public:
			Object()
				: drawingZOrder(0)/*, type(ObjectType::None)*/
			{
				sprite = nullptr;
				tilemap = nullptr;
				particleSystem = nullptr;
				useVShader = nullptr;
				useFShader = nullptr;
				blendMode = sf::BlendAlpha;
			}
			~Object()
			{
				if (sprite)
					delete sprite;
				if (tilemap)
					delete tilemap;
				if (particleSystem)
					delete particleSystem;
			}
			const sf::Int8& getDrawingZLayer() const
			{
				return drawingZOrder;
			}
			const bool& isVisible() const
			{
				return visible;
			}
			const ObjectType& getObjectType() const
			{
				return type;
			}
			const sf::BlendMode& getBlendMode() const
			{
				return blendMode;
			}
			Transformable& getTransformable() const
			{
				if (tilemap)
					return (*tilemap);
				if (sprite)
					return (*sprite);
				if (particleSystem)
					return (*particleSystem);
				sf::err() << "Object does not have a transformable!" << std::endl;
				return (*sprite);
			}
			sf::Sprite& getSprite() const
			{
				if (!sprite)
					sf::err() << "Object not a sprite!" << std::endl;
				return (*sprite);
			}
			TileMap& getTileMap() const
			{
				if (!tilemap)
					sf::err() << "Object not a tilemap!" << std::endl;
				return (*tilemap);
			}
			ParticleSystem& getParticleSystem() const
			{
				if (!particleSystem)
					sf::err() << "Object not a particle system!" << std::endl;
				return (*particleSystem);
			}
			const sf::Shader& getShader() const
			{
				return shader;
			}
			void setDrawingLayer(sf::Int8 drawingLayer)
			{
				drawingZOrder = drawingLayer;
			}
			void setVisible(bool visible)
			{
				this->visible = visible;
			}
			void setBlendMode(const sf::BlendMode& blendMode)
			{
				this->blendMode = blendMode;
			}
			template<typename T>
			void setUniform(const std::string& name, T value)
			{
				shader.setUniform(name, value);
			}

		private:
			virtual void draw(sf::RenderTarget& target, sf::RenderStates states) const
			{
				states.blendMode = blendMode;
				if (sprite)
					target.draw(*sprite, states);
				if (tilemap)
					target.draw(*tilemap, states);
				if (particleSystem)
					target.draw(*particleSystem, states);
			}
		};
		void AnalyzeShader(RenderStateObject& targetObject)
		{
			if (targetObject.type != ObjectType::Shader_F && targetObject.type != ObjectType::Shader_V)
				return;
			sf::FileInputStream load1;
			if (load1.open(targetObject.textureLocation))
			{
				auto stringToType = [](const std::string& str) {
					if (str == "bool")
						return Uniforms::DataTypes::Bool;
					if (str == "int")
						return Uniforms::DataTypes::Int;
					if (str == "float")
						return Uniforms::DataTypes::Float;
					if (str == "ivec2")
						return Uniforms::DataTypes::IVec2;
					if (str == "ivec3")
						return Uniforms::DataTypes::IVec3;
					if (str == "ivec4")
						return Uniforms::DataTypes::IVec4;
					if (str == "vec2")
						return Uniforms::DataTypes::Vec2;
					if (str == "vec3")
						return Uniforms::DataTypes::Vec3;
					if (str == "vec4")
						return Uniforms::DataTypes::Vec4;
				};
				int byteSize = 0;
				targetObject.uniforms.types.clear();
				targetObject.uniforms.uniformNames.clear();
				while (1)
				{
					std::string file;
					getString(load1, file);
					if (file.empty())
						break;
					if (file == "uniform")
					{
						getString(load1, file);
						if (file != "sampler2D")
						{
							Uniforms::DataTypes type = stringToType(file);
							byteSize += Uniforms::getTypeSize(type);
							targetObject.uniforms.types.push_back(type);
							getString(load1, file);
							if (file[file.size() - 1] == ';')
							{
								file.pop_back();
							}
							targetObject.uniforms.uniformNames.push_back(file);
						}
					}
				}
				targetObject.uniforms.byteSize = byteSize;
			}
		}
		template<typename T>
		void getNumber(sf::FileInputStream& input, T& t)
		{
			std::string s = "";
			char c;
			while (1)
			{
				input.read(&c, sizeof(char));
				if (c >= '0' && c <= '9')
					s.push_back(c);
				else
					break;
			}
			t = std::stof(s);
		}
		void getString(sf::FileInputStream& input, std::string& str)
		{
			char c;
			while (1)
			{
				input.read(&c, sizeof(char));
				if (c == ' ' || c == '\0' || c == '\n')
					break;
				else
					str.push_back(c);
			}
		}
		std::unordered_map <std::string, Object> objects;
		std::unordered_map <std::string, RenderStateObject> texture_objects;

	public:
		Level()
		{

		}
		~Level()
		{

		}
		Object& getLevelObject(const std::string& name)
		{
			return objects.at(name);
		}
		RenderStateObject& getRenderStateObject(const std::string& name)
		{
			return texture_objects.at(name);
		}
		static void LoadParticleSystem(sf::FileInputStream& file, ParticleSystem& s)
		{
			s.loadFromStream(file);
		}
		bool loadFromFile(const std::string& fileName, std::string resourceFolder)
		{
			sf::BlendMode blendModes[4] = { sf::BlendMode(sf::BlendMode::One, sf::BlendMode::Zero),
			sf::BlendMode(sf::BlendMode::SrcAlpha, sf::BlendMode::One, sf::BlendMode::Add, sf::BlendMode::One, sf::BlendMode::One, sf::BlendMode::Add),
			sf::BlendMode(sf::BlendMode::SrcAlpha, sf::BlendMode::OneMinusSrcAlpha, sf::BlendMode::Add, sf::BlendMode::One, sf::BlendMode::OneMinusSrcAlpha, sf::BlendMode::Add),
			sf::BlendMode(sf::BlendMode::DstColor, sf::BlendMode::Zero) };

			sf::FileInputStream load1;
			if (load1.open(fileName))
			{
				std::string line;
				getString(load1, line);
				if (line != ZLE_VERSION)
					return 0;
				getString(load1, line);
				int counter;
				int temp;
				float tempF;
				getNumber(load1, counter);
				std::vector<sf::Texture*> textureP;
				std::vector<RenderStateObject*> shaderP;
				textureP.resize(counter);
				shaderP.resize(counter);
				for (int i = 0; i < counter; i++)
				{
					getNumber(load1, temp);

					getString(load1, line);
					if (temp == 0)
					{
						texture_objects["None"];
						getNumber(load1, temp);
						continue;
					}
					std::string line2 = line;
					int count = 0;
					while (1)
					{
						if (line2[0] == '\\' || line2[0] == '/')
						{
							line2.erase(0, 1);
							line.erase(0, 1);
							count++;
							if (count == 2)
							{
								if (resourceFolder[resourceFolder.size() - 1] != '\\' && resourceFolder[resourceFolder.size() - 1] != '/')
									resourceFolder.push_back('/');
								line.insert(0, resourceFolder);
								break;
							}
						}
						else
						{
							line2.erase(0, 1);
							line.erase(0, 1);
						}

					}
					texture_objects[line2].type = (ObjectType)temp;
					texture_objects[line2].textureLocation = line;
					if (texture_objects[line2].type == ObjectType::Texture)
						texture_objects[line2].texture.loadFromFile(line);
					getNumber(load1, temp);
					if (texture_objects[line2].type == ObjectType::Texture)
						texture_objects[line2].texture.setSmooth(temp);
					getNumber(load1, temp);
					if (texture_objects[line2].type == ObjectType::Texture)
						texture_objects[line2].texture.setRepeated(temp);
					if (texture_objects[line2].type != ObjectType::Texture)
						AnalyzeShader(texture_objects[line2]);
					textureP[i] = &texture_objects[line2].texture;
					shaderP[i] = &texture_objects[line2];

				}
				getString(load1, line);
				getNumber(load1, counter);
				for (int i = 0; i < counter; i++)
				{
					getString(load1, line);
					getNumber(load1, temp);
					Object& o = objects[line];
					o.color.r = temp;
					getNumber(load1, temp);
					o.color.g = temp;
					getNumber(load1, temp);
					o.color.b = temp;
					getNumber(load1, temp);
					o.color.a = temp;
					getNumber(load1, temp);
					o.type = (ObjectType)temp;
					switch (o.type)
					{
					case ObjectType::Sprite:
						o.UseSprite();
						break;
					case ObjectType::Tilemap:
						o.UseTileMap();
						break;
					case ObjectType::ParticleSystem:
						o.UseParticleSystem();
						break;
					}
					getNumber(load1, temp);
					if (temp != -1)
						o.usingTexture = textureP[temp];

					getNumber(load1, temp);
					if (temp != -1)
					{
						o.useFShader = shaderP[temp];

						getString(load1, line);
						char temp2;
						o.uniformFData.resize(std::stoi(line));
						for (int i = 0; i < std::stoi(line); i++)
						{
							load1.read(&temp2, sizeof(char));
							o.uniformFData[i] = temp2;
						}
					}

					getNumber(load1, temp);
					if (temp != -1)
					{
						o.useVShader = shaderP[temp];

						getString(load1, line);

						o.uniformVData.resize(std::stoi(line));

						char temp2;
						for (int i = 0; i < std::stoi(line); i++)
						{
							load1.read(&temp2, sizeof(char));
							o.uniformVData[i] = temp2;
						}
					}
					if (o.useVShader && o.useFShader)
					{
						o.shader.loadFromFile(o.useVShader->textureLocation, o.useFShader->textureLocation);
					}
					else if (o.useFShader && !o.useVShader)
						o.shader.loadFromFile(o.useFShader->textureLocation, sf::Shader::Fragment);
					else if (!o.useFShader && o.useVShader)
						o.shader.loadFromFile(o.useVShader->textureLocation, sf::Shader::Vertex);

					getNumber(load1, temp);
					o.visible = temp;
					getNumber(load1, temp);
					o.drawingZOrder = temp;
					getNumber(load1, temp);
					o.blendMode = blendModes[temp];

					sf::Vector2f tempV;
					Transformable& t = o.getTransformable();
					getNumber(load1, tempV.x);
					getNumber(load1, tempV.y);
					t.setPosition(tempV);

					getNumber(load1, tempF);
					t.setRotation(tempF);

					getNumber(load1, tempV.x);
					getNumber(load1, tempV.y);
					t.setScale(tempV);

					getNumber(load1, tempV.x);
					getNumber(load1, tempV.y);
					t.setOrigin(tempV);

					switch (o.type)
					{
					case ObjectType::Sprite:
					{
						sf::Sprite* s = o.sprite;
						s->setColor(o.color);

						sf::IntRect tempI;
						getNumber(load1, tempI.left);
						getNumber(load1, tempI.top);
						getNumber(load1, tempI.width);
						getNumber(load1, tempI.height);
						s->setTextureRect(tempI);
						if (o.usingTexture)
							s->setTexture(*o.usingTexture);
					}
					break;
					case ObjectType::Tilemap:
					{
						TileMap* s = o.tilemap;


						getNumber(load1, temp);
						s->setTextureGap(temp);

						getNumber(load1, tempV.x);
						getNumber(load1, tempV.y);
						s->setTextureTileSize((sf::Vector2u)tempV);
						getNumber(load1, tempV.x);
						getNumber(load1, tempV.y);
						s->setTileCount((sf::Vector2u)tempV);
						getNumber(load1, tempV.x);
						getNumber(load1, tempV.y);
						s->setTileSize((sf::Vector2u)tempV);

						s->setColor(o.color);

						for (int i = 0; i < s->getTileCount().x; i++)
							for (int j = 0; j < s->getTileCount().y; j++)
							{
								getNumber(load1, temp);
								s->setTileID(sf::Vector2u(i, j), temp);
							}
						if (o.usingTexture)
							s->setTexture(o.usingTexture);
					}
					break;
					case ObjectType::ParticleSystem:
					{
						LoadParticleSystem(load1, *o.particleSystem);
						if (o.usingTexture)
							o.particleSystem->setTexture(*o.usingTexture);
					}
					break;
					}

				}
				return 1;
			}
			return 0;
		}


	private:
		virtual void draw(sf::RenderTarget& target, sf::RenderStates states) const
		{
			states.transform *= getTransform();
			for (int i = INT8_MIN; i < INT8_MAX; i++)
			{
				for (auto& n : objects)
				{
					if (!n.second.visible)
						continue;
					if (n.second.drawingZOrder == i)
					{
						if (n.second.useFShader || n.second.useVShader)
							states.shader = &n.second.shader;
						if (!n.second.useFShader && !n.second.useVShader)
							states.shader = nullptr;
						target.draw(n.second, states);
					}
				}
			}
		}
	};
}